package com.eitc.common.handler;

import com.baomidou.mybatisplus.core.handlers.MetaObjectHandler;
import com.eitc.common.exception.ServiceException;
import com.eitc.common.utils.JWTUtil;
import com.eitc.common.utils.SecurityUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.ibatis.reflection.MetaObject;
import org.springframework.context.annotation.Configuration;

import java.util.Date;

/**
 * MyMetaObjectHandler
 *
 * @author mtx
 * @date: 2024/05/30 9:45
 */
@Slf4j
@Configuration
public class MyMetaObjectHandler implements MetaObjectHandler {

    /**
     * 通用的新增字段
     */
    @Override
    public void insertFill(MetaObject metaObject) {
        //判断是否有传递该属性
        if (metaObject.hasSetter("createUser")) {
            Object createId = getFieldValByName("createUser", metaObject);
            if (createId == null) {
//                log.info("创建人字段值为空 已经填充 ");
                //开发中 创建者 修改者ID 通过Aop 或者权限框架获取当前操作用户 ，用用户ID进行填充即可
                this.strictInsertFill(metaObject, "createUser", String.class, getUserName());

            }
        }
        //判断是否有传递该属性
        if (metaObject.hasSetter("createDate")) {
            Object createTime = getFieldValByName("createDate", metaObject);
            if (createTime == null) {
//                log.info("创建时间字段值为空 已经填充 ");
                this.strictInsertFill(metaObject, "createDate", Date.class, new Date());
            }
        }
        //判断是否有传递该属性
//        if (metaObject.hasSetter("tenantId")) {
//            Object tenantId = getFieldValByName("tenantId", metaObject);
//            if (tenantId == null) {
////                log.info("租户字段值为空 已经填充 ");
//                this.strictInsertFill(metaObject, "tenantId", String.class, "000001");
//            }
//        }
        updateFill(metaObject);
    }

    /**
     * 通用的更新字段
     */
    @Override
    public void updateFill(MetaObject metaObject) {
        //判断是否有传递该属性
        if (metaObject.hasSetter("updateUser")) {
            Object modifyId = getFieldValByName("updateUser", metaObject);
            if (modifyId == null) {
//                log.info("更新人字段值为空 已经填充 ");
                this.strictUpdateFill(metaObject, "updateUser", String.class, getUserName());
            }
        }
        //判断是否有传递该属性
        if (metaObject.hasSetter("updateDate")) {
            Object modifyTime = getFieldValByName("updateDate", metaObject);
            if (modifyTime == null) {
//                log.info("更新时间字段值为空 已经填充 ");
                this.strictUpdateFill(metaObject, "updateDate", Date.class, new Date());
            }
        }
    }

    private String getUserName() {
        String username = "";
        try {
            try {
                username = SecurityUtils.getUsername();
            } catch (ServiceException e) {
                log.error("获取用户失败{}", e.getMessage());
                username = JWTUtil.getAppPhoneNumber();
            }
        } catch (Exception e) {
            log.error("获取用户失败{}", e.getMessage());
        }
        return username;
    }
}
